;; (C) 2006-2007 Pavel Mihaylov
;; modified: Ben Waldron 09/05/2006

;;; Some hacks to make LKB talk to us

(in-package :lkb)
    
;; Setting (stream-external-format *terminal-io*) to :utf-8
(setf (stream-external-format *terminal-io*) :utf-8)

(defun trollet-error-handler (condition)
   (format *error-output* "~&<error>~A</error>~&" condition)
   (abort condition))
                                              
;; call all functions through this macro
(defmacro abort-on-error (&body forms)
    `(handler-bind ((error #'trollet-error-handler))
        ;(format t "~&<output>~&")
        ,@forms
        (format t "~&</output>~&"))) 

(defun call-perl (&rest cmds)
    (setq rs (code-char 30))
    ; ASCII 31 is 'unit separator', 30 is 'record separator';
    ; good old meanings I'd like to use
    (setf sock
        (acl-socket:make-socket
            :address-family :file
            :format :text
            :type :stream
            :remote-filename (format nil "~a/~a" (sys:getenv "HOME") ".trollet/socket")))
    (setf (stream-external-format sock) :utf8)
    (format sock "~a" (first cmds))
    (loop for cmd in (rest cmds) do
        (format sock "~a~a" rs cmd))
    (force-output sock)
    (acl-socket:shutdown sock :direction :output)
    (setq result (read sock))
    (close sock)
    (cond ((equal result "NIL") (setq result nil))
          ((equal result "T") (setq result t)))
    result)


(defun show-message-window (message)
    (call-perl "message" message))

(defun ask-user-for-existing-pathname (title)
    (call-perl "existing file" title))

(defun ask-user-for-new-pathname (title input-file)
    (call-perl "new file" title input-file))


(defun strcat (&rest strings)
    (apply #'concatenate 'string strings))



(defun ask-for-strings-movable (title prompt-init-pairs 
				&optional width choices)
    ;(print prompt-init-pairs)
    ; ASCII 31 is 'unit separator', 30 is 'record separator';
    ; good old meanings I'd like to use
    (setq us (code-char 31))
   
    (setq keys "")
    (loop for p-i-p in prompt-init-pairs do
        (setq keys
            (strcat keys
                (format nil "~a~a" (car p-i-p) us))))

    (setq vals "")
    (loop for p-i-p in prompt-init-pairs for count from 0 do
        (setq vals
            (strcat vals
             (format nil "~a~a"
                (cond
                    ((and (consp (rest p-i-p))
                          (eq (second p-i-p) :TYPEIN-MENU))
                        (third p-i-p))
                    (t
                     (cdr p-i-p)))
                us))))

    (setq choicestr "")
    ; Limit number of choice strings
    (if (< (length choices) 1000)
        (loop for choice in choices do
            (setq glbpos (search "GLBTYPE" (format nil "~a" choice)))
            (unless (and glbpos (= glbpos 0))
                (setq choicestr
                    (strcat choicestr
                        (format nil "~a~a" choice us))))))

    (setq result (call-perl "strings movable" title keys vals width choicestr))
    result)



(defun source-available-p (thing)
    (format t "foo~&")     
    (ignore-errors (excl:source-file thing :lkb)))

(defun edit-file (thing file)
    (call-perl "show source" thing file))



(in-package :clim-user)

;; No thanks, keep it out of the top window
(defun invoke-with-output-to-top (body)
  (unwind-protect
        (funcall body)))

;; get rid of CLIM window
(defun destroy-clim-window nil
    (execute-frame-command *lkb-top-frame* '(com-close-to-replace)))



